@@ -4,7 +4,7 @@ Utils for time series statistics
 """
 
 import math
-from typing import Tuple, Optional, Union
+from typing import Tuple, Optional, List, Union
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -13,6 +13,7 @@ from scipy.signal import argrelmax
 from statsmodels.tsa.seasonal import seasonal_decompose
 from statsmodels.tsa.stattools import acf
 
+from warnings import warn
 from ..logging import raise_log, get_logger, raise_if_not
 from ..timeseries import TimeSeries
 from .missing_values import fill_missing_values
@@ -288,7 +289,7 @@ def plot_acf(ts: TimeSeries,
     for i in range(1, max_lag + 1):
         stats.append(_bartlett_formula(r[1:], i, len(ts)))
 
-    if (axis is None):
+    if axis is None:
         plt.figure(figsize=fig_size)
         axis = plt
 
@@ -305,9 +306,81 @@ def plot_acf(ts: TimeSeries,
     axis.plot((0, max_lag + 1), (0, 0), color='black')
 
 
+def plot_hist(data: Union[TimeSeries, List[float], np.ndarray],
+              bins: Optional[Union[int, np.ndarray, List[float]]] = None,
+              density: bool = False,
+              title: Optional[str] = None,
+              fig_size: Optional[Tuple[int, int]] = None,
+              ax: Optional[plt.axis] = None) -> None:
+    """ This function plots the histogram of values in a TimeSeries instance or an array-like.
+
+    All types of TimeSeries are supported (uni-, multivariate, deterministic, stochastic).
+    Depending on the number of components in `data`, up to four histograms can be plotted on one figure.
+    All stochastic samples will be displayed with the corresponding component.
+
+    If `data` is an array-like, ALL values will be displayed in the same histogram.
+
+    Parameters
+    ----------
+    data
+        TimeSeries instance or an array-like from which to plot the histogram.
+    bins
+        Optionally, either an integer value for the number of bins to be displayed
+        or an array-like of floats determining the position of bins.
+    density
+        bool, if `density` is set to True, the bin counts will be converted to probability density
+    title
+        The title of the figure to be displayed
+    fig_size
+        The size of the figure to be displayed.
+    ax
+        Optionally, an axis object to plot the histogram on.
+    """
+
+    n_plots_max = 4
+
+    if isinstance(data, TimeSeries):
+        if len(data.components) > n_plots_max:
+            logger.warning("TimeSeries contains more than 4 components. Only the first 4 components will be displayed.")
+
+        components = list(data.components[:n_plots_max])
+        values = data[components].all_values(copy=False).flatten(order='F')
+    else:
+        values = data if isinstance(data, np.ndarray) else np.array(data)
+        if len(values.shape) > 1:
+            logger.warning("Input array-like data with `dim>1d` will be flattened and displayed in one histogram.")
+
+        components = ['Data']
+        values = values.flatten(order='F')
+
+    # compute the number of columns and rows for subplots depending on shape of input data
+    n_components = len(components)
+    n_cols = 1 if n_components == 1 else 2
+    n_rows = math.ceil(n_components / n_cols)
+
+    title = 'Histogram' if title is None else title
+    if ax is None:
+        fig = plt.figure(constrained_layout=True, figsize=fig_size)
+        gs = fig.add_gridspec(n_rows, n_cols)
+        fig.suptitle(title)
+        ax_all = [fig.add_subplot(gs[i]) for i in range(n_components)]
+    else:
+        if n_components > 1:
+            logger.warning("Only the first component is plotted when calling plot_hist() with a given `ax`")
+        ax.set_title(title)
+        ax_all = [ax]
+
+    n_entries = len(values) // n_components
+    for i, label, ax_i in zip(range(n_components), components, ax_all):
+        ax_i.hist(values[i * n_entries:(i + 1) * n_entries], bins=bins, density=density, label=label)
+        ax_i.set_xlabel('value')
+        ax_i.set_ylabel('count' if not density else 'probability density')
+        ax_i.legend()
+
+
 def plot_residuals_analysis(residuals: TimeSeries,
                             num_bins: int = 20,
-                            fill_nan: bool = True):
+                            fill_nan: bool = True) -> None:
     """ Plots data relevant to residuals.
 
     This function takes a univariate TimeSeries instance of residuals and plots their values,
@@ -340,12 +413,12 @@ def plot_residuals_analysis(residuals: TimeSeries,
     ax1.set_ylabel('value')
     ax1.set_title('Residual values')
 
-    # plot distribution
+    # plot histogram and distribution
     res_mean, res_std = np.mean(residuals.univariate_values()), np.std(residuals.univariate_values())
     res_min, res_max = min(residuals.univariate_values()), max(residuals.univariate_values())
     x = np.linspace(res_min, res_max, 100)
     ax2 = fig.add_subplot(gs[1:, 1:])
-    ax2.hist(residuals.univariate_values(), bins=num_bins)
+    plot_hist(residuals, bins=num_bins, ax=ax2)
     ax2.plot(x, norm(res_mean, res_std).pdf(x) * len(residuals) * (res_max - res_min) / num_bins)
     ax2.yaxis.set_major_locator(plt.MaxNLocator(integer=True))
     ax2.set_title('Distribution')
